@@ -1,6 +1,6 @@
 import io
 from enum import IntEnum
-from typing import Any, Dict, List, Tuple, Optional
+from typing import Any, Dict, List, Tuple, Optional, NamedTuple
 from datetime import datetime
 from collections import defaultdict
 
@@ -39,14 +39,20 @@ from Orange.data import Table, Domain, StringVariable
 from Orange.widgets import gui, widget, settings
 from Orange.widgets.widget import Msg, Output, StateInfo, OWComponent
 from Orange.data.pandas_compat import table_from_frame
+from Orange.widgets.credentials import CredentialManager
 from Orange.widgets.utils.concurrent import TaskState, ConcurrentWidgetMixin
 from Orange.widgets.utils.itemmodels import PyTableModel
 
-from orangecontrib.bioinformatics.resolwe import ResolweAPI, ResolweAuthException, connect
+from orangecontrib.bioinformatics.resolwe import ResolweAPI, ResolweAuthException, ResolweDataObjectsNotFound, connect
 from orangecontrib.bioinformatics.ncbi.gene import GeneMatcher
 from orangecontrib.bioinformatics.preprocess import ZScore, LogarithmicScale, QuantileTransform, QuantileNormalization
 from orangecontrib.bioinformatics.ncbi.taxonomy import species_name_to_taxid
-from orangecontrib.bioinformatics.resolwe.resapi import DEFAULT_URL, RESOLWE_URLS, SAMPLE_DESCRIPTOR_LABELS
+from orangecontrib.bioinformatics.resolwe.resapi import (
+    DEFAULT_URL,
+    RESOLWE_URLS,
+    SAMPLE_DESCRIPTOR_LABELS,
+    CREDENTIAL_MANAGER_SERVICE,
+)
 from orangecontrib.bioinformatics.widgets.utils.data import TableAnnotation
 
 PAST_HOUR = relativedelta(hours=-1)
@@ -194,6 +200,7 @@ class CollapsibleFilterComponent(OWComponent, QObject):
         self.collapsible_components.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self.collapsible_components.setFrameShape(QFrame.NoFrame)
 
+        box = gui.widgetBox(parent_component, margin=0)
         box.layout().addWidget(self.toggle_button)
         box.layout().addWidget(self.collapsible_components)
 
@@ -439,6 +446,8 @@ class NormalizationComponent(OWComponent, QObject):
 class SignInForm(QDialog):
     def __init__(self, flags, *args, **kwargs):
         super().__init__(flags, *args, **kwargs)
+        self.cm: CredentialManager = CredentialManager(CREDENTIAL_MANAGER_SERVICE)
+
         self.setWindowTitle('Sign in')
         self.setFixedSize(400, 250)
 
@@ -493,8 +502,8 @@ class SignInForm(QDialog):
         self.error_msg.hide()
 
         server = self.server_cb.currentText()
-        username = self.username_line_edit.text()
-        password = self.password_line_edit.text()
+        username = self.cm.username if self.cm.username else self.username_line_edit.text()
+        password = self.cm.password if self.cm.password else self.password_line_edit.text()
 
         if not server:
             self.server_cb_label.setStyleSheet('color:red')
@@ -514,6 +523,8 @@ class SignInForm(QDialog):
             self.error_msg.show()
             return
 
+        self.cm.username = username
+        self.cm.password = password
         self.accept()
 
 
@@ -598,7 +609,37 @@ class GenialisExpressionsModel(PyTableModel):
         self.wrap([model_row(result) for result in collections])
 
 
-def runner(res: ResolweAPI, data_objects: List[Data], exp_type: str, species: str, state: TaskState) -> Table:
+class Expression(NamedTuple):
+    type: str
+    name: str
+
+
+class Process(NamedTuple):
+    type: str
+    name: str
+
+
+class InputAnnotation(NamedTuple):
+    source: str
+    species: str
+    build: str
+
+
+class DataOutputOptions(NamedTuple):
+    expression: Tuple[Expression]
+    process: Tuple[Process]
+    input_annotation: Tuple[InputAnnotation]
+
+
+def runner(
+    res: ResolweAPI,
+    data_objects: List[Data],
+    options: DataOutputOptions,
+    exp_type: int,
+    proc_type: int,
+    input_annotation: int,
+    state: TaskState,
+) -> Table:
     data_frames = []
     metadata = defaultdict(list)
 
@@ -610,11 +651,25 @@ def runner(res: ResolweAPI, data_objects: List[Data], exp_type: str, species: st
 
         metadata['sample_name'].append([sample.name])
 
+    exp_type = file_output_field = options.expression[exp_type].type
+    proc_type = options.process[proc_type].type
+    source = options.input_annotation[input_annotation].source
+    species = options.input_annotation[input_annotation].species
+    build = options.input_annotation[input_annotation].build
+
+    # apply filters
+    data_objects = [obj for obj in data_objects if obj.process.type == proc_type]
+    data_objects = [
+        obj
+        for obj in data_objects
+        if obj.output['source'] == source and obj.output['species'] == species and obj.output['build'] == build
+    ]
     if exp_type != 'rc':
-        output_field = 'exp'
+        file_output_field = 'exp'
         data_objects = [obj for obj in data_objects if obj.output['exp_type'] == exp_type]
-    else:
-        output_field = 'rc'
+
+    if not data_objects:
+        raise ResolweDataObjectsNotFound
 
     step, steps = 0, len(data_objects) + 3
 
@@ -629,7 +684,7 @@ def runner(res: ResolweAPI, data_objects: List[Data], exp_type: str, species: st
         parse_sample_descriptor(data_object.sample)
         metadata['expression_type'].append([exp_type.upper()])
 
-        response = res.get_expressions(data_object.id, data_object.output[output_field]['file'])
+        response = res.get_expressions(data_object.id, data_object.output[file_output_field]['file'])
         with io.BytesIO() as f:
             f.write(response.content)
             f.seek(0)
@@ -657,7 +712,7 @@ def runner(res: ResolweAPI, data_objects: List[Data], exp_type: str, species: st
     state.set_status('Matching genes ...')
     tax_id = species_name_to_taxid(species)
     gm = GeneMatcher(tax_id)
-    gm.match_table_attributes(table)
+    table = gm.match_table_attributes(table, rename=True)
     table.attributes[TableAnnotation.tax_id] = tax_id
     table.attributes[TableAnnotation.gene_as_attr_name] = True
     table.attributes[TableAnnotation.gene_id_attribute] = 'Entrez ID'
@@ -682,6 +737,12 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
     exp_type: int
     exp_type = settings.Setting(None, schema_only=True)
 
+    proc_type: int
+    proc_type = settings.Setting(None, schema_only=True)
+
+    input_annotation: int
+    input_annotation = settings.Setting(None, schema_only=True)
+
     auto_commit: bool
     auto_commit = settings.Setting(False, schema_only=True)
 
@@ -690,33 +751,49 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
 
     class Warning(widget.OWWidget.Warning):
         no_expressions = Msg('Expression data objects not found.')
+        no_data_objects = Msg('No expression data matches the selected filtering options.')
         unexpected_feature_type = Msg('Can not import expression data, unexpected feature type "{}".')
         multiple_feature_type = Msg('Can not import expression data, multiple feature types found.')
 
     def __init__(self):
         super().__init__()
         ConcurrentWidgetMixin.__init__(self)
-        self._res = connect(url=DEFAULT_URL)
+
+        self._res = None
+        self._data_objects: Optional[List[Data]] = None
+        self.data_output_options: Optional[DataOutputOptions] = None
+        self.data_table: Optional[Table] = None
 
         # Control area
-        box = gui.widgetBox(self.controlArea, 'Sign in')
-        self.user_info = gui.label(box, self, '')
-        self.server_info = gui.label(box, self, '')
+        self.info_box = gui.widgetLabel(gui.widgetBox(self.controlArea, "Info", addSpace=True), 'No data on output.')
 
-        box = gui.widgetBox(box, orientation=Qt.Horizontal)
-        self.sign_in_btn = gui.button(box, self, 'Sign in', callback=self.sign_in, autoDefault=False)
-        self.sign_out_btn = gui.button(box, self, 'Sign out', callback=self.sign_out, autoDefault=False)
-
-        self.exp_type_box = gui.widgetBox(self.controlArea, 'Expression type')
+        self.exp_type_box = gui.widgetBox(self.controlArea, 'Expression Type')
         self.exp_type_options = gui.radioButtons(
-            self.exp_type_box, self, 'exp_type', callback=self.on_exp_type_changed
+            self.exp_type_box, self, 'exp_type', callback=self.on_data_output_option_changed
+        )
+
+        self.proc_type_box = gui.widgetBox(self.controlArea, 'Process Name')
+        self.proc_type_options = gui.radioButtons(
+            self.proc_type_box, self, 'proc_type', callback=self.on_data_output_option_changed
+        )
+
+        self.input_anno_box = gui.widgetBox(self.controlArea, 'Expression source')
+        self.input_anno_options = gui.radioButtons(
+            self.input_anno_box, self, 'input_annotation', callback=self.on_data_output_option_changed
         )
-        self.set_exp_type_options()
 
         self.norm_component = NormalizationComponent(self, self.controlArea)
         self.norm_component.options_changed.connect(self.on_normalization_changed)
 
         gui.rubber(self.controlArea)
+        box = gui.widgetBox(self.controlArea, 'Sign in')
+        self.user_info = gui.label(box, self, '')
+        self.server_info = gui.label(box, self, '')
+
+        box = gui.widgetBox(box, orientation=Qt.Horizontal)
+        self.sign_in_btn = gui.button(box, self, 'Sign In', callback=self.sign_in, autoDefault=False)
+        self.sign_out_btn = gui.button(box, self, 'Sign Out', callback=self.sign_out, autoDefault=False)
+
         self.commit_button = gui.auto_commit(self.controlArea, self, 'auto_commit', '&Commit', box=False)
         self.commit_button.button.setAutoDefault(False)
 
@@ -743,10 +820,7 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
         self.pagination_component = PaginationComponent(self, self.mainArea)
         self.pagination_component.options_changed.connect(self.update_collections_view)
 
-        self.data_objects: Optional[List[Data]] = None
-        self.data_table: Optional[Table] = None
-        self.update_collections_view()
-        self.update_user_status()
+        self.sign_in(silent=True)
 
     def __invalidate(self):
         self.data_objects = None
@@ -755,28 +829,58 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
         self.Warning.multiple_feature_type.clear()
         self.Warning.unexpected_feature_type.clear()
         self.info.set_output_summary(StateInfo.NoOutput)
+        self.update_info_box()
 
-    def set_exp_type_options(self, options: List[str] = None) -> None:
-        self.clear_exp_type_options()
-        gui.appendRadioButton(self.exp_type_options, 'Read Counts', disabled=not bool(options))
+    def set_input_annotation_options(self) -> None:
+        for btn in self.input_anno_options.buttons:
+            btn.deleteLater()
+        self.input_anno_options.buttons = []
 
-        for option in options if options is not None else []:
-            gui.appendRadioButton(self.exp_type_options, option)
+        if not self.data_output_options:
+            return
 
-        if len(self.exp_type_options.buttons) > 1:
-            self.exp_type = 1
+        for source, species, build in self.data_output_options.input_annotation:
+            tooltip = f'{source}, {species}, {build}'
+            text = f'{species}, {build}'
+            gui.appendRadioButton(self.input_anno_options, text, tooltip=tooltip)
 
-    def clear_exp_type_options(self) -> None:
+        if len(self.input_anno_options.buttons):
+            self.input_annotation = 0
+
+    def set_proc_type_options(self) -> None:
+        for btn in self.proc_type_options.buttons:
+            btn.deleteLater()
+        self.proc_type_options.buttons = []
+
+        if not self.data_output_options:
+            return
+
+        for proc_type, proc_name in self.data_output_options.process:
+            gui.appendRadioButton(self.proc_type_options, proc_name, tooltip=proc_type)
+
+        if len(self.proc_type_options.buttons):
+            self.proc_type = 0
+
+    def set_exp_type_options(self) -> None:
         for btn in self.exp_type_options.buttons:
             btn.deleteLater()
         self.exp_type_options.buttons = []
 
+        if not self.data_output_options:
+            return
+
+        for _, exp_name in self.data_output_options.expression:
+            gui.appendRadioButton(self.exp_type_options, exp_name)
+
+        if len(self.exp_type_options.buttons) > 1:
+            self.exp_type = 1
+
     @property
     def res(self):
         return self._res
 
     @res.setter
-    def res(self, value):
+    def res(self, value: ResolweAPI):
         if isinstance(value, ResolweAPI):
             self._res = value
             self.update_user_status()
@@ -784,27 +888,99 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
             self.__invalidate()
             self.Outputs.table.send(None)
 
+    @property
+    def data_objects(self):
+        return self._data_objects
+
+    @data_objects.setter
+    def data_objects(self, data_objects: Optional[List[Data]]):
+        self._data_objects = data_objects
+        self.data_output_options = self._available_data_output_options()
+
+    def _available_data_output_options(self) -> Optional[DataOutputOptions]:
+        """
+        Traverse the data objects in the selected collection and store the
+        information regarding available expression types, process types and
+        input annotations used in the creation of the data object.
+
+        The method returns a named tuple (`DataOutputOptions`) which used for
+        creating radio buttons in the control area.
+        """
+        if not self.data_objects:
+            return
+
+        expression_types = sorted({data.output['exp_type'] for data in self.data_objects})
+        expression_types = (Expression('rc', 'Read Counts'),) + tuple(
+            Expression(exp_type, exp_type) for exp_type in expression_types
+        )
+
+        process_types = sorted({(data.process.type, data.process.name) for data in self.data_objects})
+        process_types = tuple(Process(proc_type, proc_name) for proc_type, proc_name in process_types)
+
+        input_annotations = sorted(
+            {(data.output['source'], data.output['species'], data.output['build']) for data in self.data_objects}
+        )
+        input_annotations = tuple(
+            InputAnnotation(source, species, build) for source, species, build in input_annotations
+        )
+
+        return DataOutputOptions(
+            expression=expression_types, process=process_types, input_annotation=input_annotations
+        )
+
     def update_user_status(self):
         user = self.res.get_currently_logged_user()
 
         if user:
             user_info = f"{user[0].get('first_name', '')} {user[0].get('last_name', '')}".strip()
             user_info = f"User: {user_info if user_info else user[0].get('username', '')}"
+            self.sign_in_btn.setEnabled(False)
             self.sign_out_btn.setEnabled(True)
         else:
             user_info = 'User: Anonymous'
+            self.sign_in_btn.setEnabled(True)
             self.sign_out_btn.setEnabled(False)
 
         self.user_info.setText(user_info)
         self.server_info.setText(f'Server: {self.res.url[8:]}')
 
-    def sign_in(self):
+    def update_info_box(self):
+
+        if self.data_table:
+            total_genes = len(self.data_table.domain.attributes)
+            known_genes = len([col for col in self.data_table.domain.attributes if len(col.attributes)])
+
+            info_text = (
+                '{} genes on output\n'
+                '{} genes match Entrez database\n'
+                '{} genes with match conflicts\n'.format(total_genes, known_genes, total_genes - known_genes)
+            )
+
+        else:
+            info_text = 'No data on output.'
+
+        self.info_box.setText(info_text)
+
+    def sign_in(self, silent=False):
         dialog = SignInForm(self)
-        if dialog.exec_():
+
+        if silent:
+            dialog.sign_in()
+            if dialog.resolwe_instance is not None:
+                self.res = dialog.resolwe_instance
+            else:
+                self.res = connect(url=DEFAULT_URL)
+
+        if not silent and dialog.exec_():
             self.res = dialog.resolwe_instance
 
     def sign_out(self):
+        # Use public credentials when user signs out
         self.res = connect(url=DEFAULT_URL)
+        # Remove username and
+        cm = CredentialManager(CREDENTIAL_MANAGER_SERVICE)
+        del cm.username
+        del cm.password
 
     def on_filter_changed(self):
         self.pagination_component.reset_pagination()
@@ -874,25 +1050,30 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
 
         if self.norm_component.quantile_transform:
             axis = self.norm_component.quantile_transform_axis
-            quantiles = min(table.X.shape[int(not axis)], 100)
+            quantiles = table.X.shape[int(not axis)]
             distribution = QuantileTransformDist.values()[self.norm_component.quantile_transform_dist]
             table = QuantileTransform(axis=axis, n_quantiles=quantiles, output_distribution=distribution)(table)
 
         return table
 
     def commit(self):
+        self.Warning.no_data_objects.clear()
         self.cancel()
 
         if self.data_objects and not self.data_table:
-            exp_type: str = 'rc' if self.exp_type == 0 else self.exp_type_options.group.checkedButton().text()
-            collection_species: str = self.get_selected_row_data(TableHeader.species)
-
-            self.start(runner, self.res, self.data_objects, exp_type, collection_species)
-
+            self.start(
+                runner,
+                self.res,
+                self.data_objects,
+                self.data_output_options,
+                self.exp_type,
+                self.proc_type,
+                self.input_annotation,
+            )
         else:
             self.Outputs.table.send(self.normalize(self.data_table))
 
-    def on_exp_type_changed(self):
+    def on_data_output_option_changed(self):
         self.data_table = None
 
         if self.data_objects:
@@ -910,7 +1091,9 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
             return
 
         self.data_objects = self.res.get_expression_data_objects(collection_id)
-        self.set_exp_type_options(list({data.output['exp_type'] for data in self.data_objects}))
+        self.set_exp_type_options()
+        self.set_proc_type_options()
+        self.set_input_annotation_options()
 
         if not self.data_objects:
             self.Warning.no_expressions()
@@ -945,10 +1128,21 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
     def on_done(self, table: Table):
         if table:
             samples, genes = table.X.shape
-            self.info.set_output_summary(f'Samples: {samples} Genes: {genes}')
             self.data_table = table
+            self.info.set_output_summary(f'Samples: {samples} Genes: {genes}')
+            self.update_info_box()
             self.Outputs.table.send(self.normalize(table))
 
+    def on_exception(self, ex):
+        if isinstance(ex, ResolweDataObjectsNotFound):
+            self.Warning.no_data_objects()
+            self.Outputs.table.send(None)
+            self.data_table = None
+            self.info.set_output_summary(StateInfo.NoOutput)
+            self.update_info_box()
+        else:
+            raise ex
+
     def on_partial_result(self, result: Any) -> None:
         pass
 
@@ -956,6 +1150,9 @@ class OWGenialisExpressions(widget.OWWidget, ConcurrentWidgetMixin):
         self.shutdown()
         super().onDeleteWidget()
 
+    def sizeHint(self):
+        return QSize(1280, 620)
+
 
 if __name__ == "__main__":
     from orangewidget.utils.widgetpreview import WidgetPreview
